 *** CSAE coders corner sample do file and dataset
 
*/

{ /* Preable */
	cap log close
	set more off
	set rmsg on
	set matsize 11000
	capture ssc install  ppml_panel_sg
	capture ssc install hdfe
	capture ssc install unique
	clear
	
	// set path to where you have saved the data
	cd "E:\Dropbox\OneDrive\DPhil\CSAE coders corner"
}

// load sample data 
use sample_data.dta

	/* Data set description
		- This is a trade-flow data set, as such every origin country (exporter) has a destination country
		for every year (2000 to 2010), thus an ijt data set. 
		- Note, this is a quasi made up dataset, results have no importance or significance.
		- Common issue is that free trade agreements maybe endogenous - we look to instrument this
			with share_border, and calculate the SE by bootstrapping.
		- I implement a theoreticaly-consistent gravity model using PPML see http://personal.lse.ac.uk/tenreyro/lgw.html
	*/


	// prog1: two step estimation procedure, adjust this program to the model you are considering
	capture program drop ppml_panel_iv_est
	program define ppml_panel_iv_est, rclass 
		capture drop fta_cf_ppml fta_hat
		
		* first stage
		ppml_panel_sg fta distance share_border, nopair exp(origin_cnty) imp(dest_cnty) year(year) clus(distance) predict(fta_hat)
		gen fta_cf_ppml = fta - fta_hat // constructed control function
		* second stage
		ppml_panel_sg trade distance fta fta_cf_ppml, nopair exp(origin_cnty) imp(dest_cnty) year(year) clus(distance)
	
	end
		
		
	// prog2: bootstrapping procedure
	capture program drop ppml_panel_iv_boot
	program define ppml_panel_iv_boot, rclass
		preserve 
		capture drop sample_panel
			egen sample_panel = group(origin dest) // set bootstrap sample params
			bsample, cl(sample_panel) idcluster(new_panel_id) // take bootstrap sample (w/ replacement)
			// redefine dest identifier such that origin-dest-year uniquely identify observations
			// note: it doesn't matter that the same county will be given different identifiers as these
			// 			are only used for dest-year FE
				unique dest
				local N = r(unique) // make sure that each synthetic identifier isn't accidentally the same as an existing one
				bysort origin year (dest): replace dest = dest + _n + `N'

			ppml_panel_iv_est // run the two step model

			mat b1 = e(b)
		
			return matrix b = b1
			
		restore
	end
		
	// prog3: main prog
	capture program drop ppml_panel_iv
	program define ppml_panel_iv, rclass
		di "Starting initial estimation"
		qui{ 
			/// run the inital estimation, two step command.
			ppml_panel_iv_est
			}
		local n = e(N) // number of observations
		// save initial SE estimates 
		mat b0 = e(b)
		
		di "Finished initial estimation, starting simulations"
		simulate , reps(50) seed(150195) : ppml_panel_iv_boot // simulate the bootstrap results
		* se = r(SE)
		
		di "Finished simulations, starting bstat"
		bstat , stat(b0) n(`n')
		
		di "Finished :)"
	end
	
// Run your program

ppml_panel_iv
